# Modern Applied Biostatistical Methods Using S-PLUS
# S. Selvin, Oxford, 1998
#
# Chapter 5: Estimation
#
# Time-stamp: <2008-08-02 10:28:49 chl>

                                        # Fig. 5.1
###
### not in the book

###


f <- function(x) { x^3+x-12 }
# initial starting value
x <- 1
for (i in 1:50) {
  if (abs(f(x))<0.00001) break
  cat(" f(x) = ",round(f(x),3),"  x = ",round(x,3),"\n")
  x <- x-f(x)/(3*x^2+1)
}

cat(" f(x) = 0 chen x = ",round(x,3),"\n")

D(expression(x^3+x-12),"x")
D(expression(sin(t)),"t")
D(expression(1-exp(-m*x)),"x")

uniroot(function(x){x^3+x-12},interval=c(0,3))$root
uniroot(f,interval=c(0,3))$root

# log-likelihood -- Poisson distribution
like <- function(lam,n,k,m,x)
  { -(n-k)*lam+log(lam)*sum(x[x<m])+k*log(1-ppois(m-1,lam)) }

x <- rep(0:3,c(26,50,58,69))
n <- length(x)
table(x)
# number of censored values
k <- 69
# censored at m
m <- 3
                                        # Fig. 5.2
# plot the likelihood function
ltemp <- seq(0.1,10,0.1)
lhood <- like(ltemp,n,k,m,x)
plot(ltemp,lhood,type="l",xlab="parameter values",ylab="log-likelihood",main="Log-likelihood function\n of the censored Poisson distribution")

# initial value
est <- mean(x)
dp <- 0.001
for (i in 1:10) {
  d1 <- (like(est+dp,n,k,m,x)-like(est,n,k,m,x))/dp
  d2 <- (like(est+2*dp,n,k,m,x)-2*like(est+dp,n,k,m,x)+like(est,n,k,m,x))/dp^2
  est <- est-d1/d2
  if (abs(d1)<0.0001) break
}
se <- sqrt(-1/d2)
cat("est = ",round(est,3)," se = ",round(se,4),"\n")

dlike <- function(lam,n,k,m,x)
  { -(n-k)+sum(x[x<m])/lam+k*dpois(m-1,lam)/(1-ppois(m-1,lam)) }

uniroot(dlike,interval=c(1,3),n=203,k=69,m=3,x=x,keep.xy=T)$root

                                        # Fig. 5.3
###
### not in the book

###


# estimation of p and d from twin data
mm <- 32
mf <- 41
ff <- 36
x <- c(mm,mf,ff)
# log-likelihood function
like <- function(x,p,d) {
  m <- 1-d
  q <- 1-p
  x[1]*log(p*m+p^2*d)+x[2]*log(2*p*q*d)+x[3]*log(q*m+q^2*d)
}
dx <- 0.001
# initial values
d <- 0.6
p <- 0.4
etemp <- NULL
for (i in 1:10) {
  f0 <- like(x,p,d)
  # approximate first partial derivatives
  f1 <- like(x,p+dx,d)
  f2 <- like(x,p,d+dx)
  b1 <- (f1-f0)/dx
  b2 <- (f2-f0)/dx
  # approximate second partial derivatives
  f11 <- like(x,p+2*dx,d)
  f22 <- like(x,p,d+2*dx)
  f12 <- like(x,p+dx,d+dx)
  g11 <- (f11-2*f1+f0)/dx^2
  g22 <- (f22-2*f2+f0)/dx^2
  g12 <- (f12-f1-f2+f0)/dx^2
  est <- rbind(p,d)
  b <- cbind(b1,b2)
  g <- cbind(c(g11,g12),c(g12,g22))
  etemp <- est-solve(g)%*%t(b)
  p <- etemp[1]
  d <- etemp[2]
  if (sum(abs(b))<0.000001) break
}
cat("p = ",round(p,3),"d = ",round(d,3),"\n")
# variance/covariance
gtemp <- -solve(g)
round(gtemp,6)

mu <- 4
sd <- 2
n <- 50
x <- rnorm(n,mu,sd)
round(mean(x),3)
round(sqrt(((n-1)/n)*var(x)),3)

like <- function(x,m,s) { -n*log(s)-0.5*sum(((x-m)/s)^2) }

gtemp <- -solve(g)
round(gtemp,3)

repx <- sample(x,length(x),replace=T)
repx <- sample(x,replace=T)

x <- c(5,8,14,6,2)
x
sample(x,replace=T)
sample(x,replace=T)
sample(x,replace=T)
sample(x,replace=T)
sample(x,replace=T)
sample(x,replace=T)

n <- 30
x <- rnorm(n)
se <- sqrt(var(x)/n)
cat(" mean = ",round(mean(x),3)," variance = ",round(var(x),3)," se = ",round(se,3),"\n")

# x contains n = 30 random standard normal values
iter <- 5000
x1 <- NULL
for (i in 1:iter) { x1[i] <- mean(x[ceiling(n*runif(n))]) }
cat(" mean = ",round(mean(x1),4)," standard error = ",round(sqrt(var(x1)),4),"\n")
xtemp <- matrix(sample(x,n*iter,replace=T),n,iter)
x2 <- apply(xtemp,2,mean)
cat(" mean = ",round(mean(x2),4)," standard error = ",round(sqrt(var(x2)),4),"\n")

                                        # Fig. 5.4
###
### not in the book

###

# x contains the n sampled data values
###
### Note:
### `fcn` is a function to be defined by the user.
### R already has a function named boot() in the MASS package, so it's not a good
### idea to define a variable having the same name.
###
temp <- matrix(sample(x,n*iter,replace=T),n,iter)
boot <- apply(temp,2,fcn)
cat(" mean = ",round(mean(boot),4)," standard error = ",round(sqrt(var(boot)),4),"\n")

# the vector d contains the n=9 sample distances
d <- c(43.64,50.67,33.56,27.75,43.35,29.56,38.83,35.95,20.01)
fcn <- function(x) { length(x)/sum(1/x) }
iter <- 2000
n <- length(x)
temp <- matrix(sample(d,n*iter,replace=T),n,iter)
boot <- apply(temp,2,fcn)
cat(" mean = ",round(mean(boot),4)," standard error = ",round(sqrt(var(boot)),4),"\n")

quantile(boot)
quantile(boot,c(0.025,0.975))

# bootstrap estimates and standard error for relative risk
a <- 178
b <- 79
na <- 1589
nb <- 1565
# x = type-A, y = type-B, 0 = no chd and 1 = chd
x <- c(rep(1,a),rep(0,na-a))
y <- c(rep(1,b),rep(0,nb-b))
rr <- NULL
for (i in 1:iter) {
  tempx <- x[ceiling(na*runif(na))]
  tempy <- y[ceiling(nb*runif(nb))]
  rr[i] <- (sum(tempx)/na)/(sum(tempy)/nb)
}
mean(rr)
# 95% confidence intervals
quantile(rr,c(0.025,0.975))

# parametric estimate
n <- 25
x <- rnorm(n)
median(x)
# estimated variance
sqrt(1.571/n)
# bootstrap estimate
iter <- 5000
d <- matrix(sample(x,n*iter,replace=T),n,iter)
xx <- apply(d,2,median)
cat(" median = ",round(mean(xx),4)," standard error = ",round(sqrt(var(xx)),4),"\n")

                                        # Fig. 5.5
###
### not in the book

###


                                        # Fig. 5.6
###
### not in the book

###

n <- length(age)
iter <- 2000
age0 <- 50
est1 <- NULL
est2 <- NULL
est3 <- NULL
for (i in 1:iter) {
  v <- ceiling(n*runif(n))
  xtemp <- age[v]
  ytemp <- sbp[v]
  b <- lsfit(xtemp,ytemp)$coef
  est1[i] <- b[1]
  est2[i] <- b[2]
  est3[i] <- b[1]+b[2]*age0
}
se1 <- sqrt(var(est1))
cat(" intercept = ",round(mean(est1),3)," se1 = ",round(se1,3),"\n")
cat(" slope = ",round(mean(est2),3)," se2 = ",round(se2,3),"\n")
se3 <- sqrt(var(est3))
cat(" y=sbp at age 50 = ",round(mean(est3),3)," se3 = ",round(se3,3),"\n")

                                        # Fig. 5.7
###
### not in the book

###

# estimation of the residuals
otemp <- glm(sbp~age)
b <- otemp$coefficients
e <- sbp-(b[1]+b[2]*age)
# bootstrap estimates of the slope
iter <- 2000
bb <- NULL
for (i in 1:iter) {
  v <- ceiling(n*runif(n))
  yy <- b[1]+b[2]*age+e[v]
  bb[i] <- lsfit(age,yy)$coef[2]
}
mean(bb)
sqrt(var(bb))

# bootstrap estimate and 95% confidence interval for R^2
dframe <- read.table("bwt.data",header=T)
attach(dframe)
x <- cbind(ht,wt,gain)
y <- bwt
otemp <- glm(y~x,family=gaussian)
r2 <- cor(predict(otemp),y)^2
cat("squared multiple correlation coefficient =",round(r2,4),"\n")
# bootstrap estimation
iter <- 1000
r <- NULL
for (i in 1:iter) {
  v <- ceiling(nrow(x)*runif(nrow(x)))
  xtemp <- x[v,]
  ytemp <- y[v]
  otemp <- glm(ytemp~xtemp,family=gaussian)
  r[i] <- cor(predict(otemp),ytemp)^2
}
rbar <- mean(r)
se <- sqrt(var(r))
cat(" bootstrap R-squared = ",round(rbar,4),"  se = ",round(se,4),"\n")
# quantile: 95% limits
round(quantile(r,c(0.025,0.975)),4)
# normal based 95% confidence limits
lower <- rbar-1.96*se
upper <- rbar+1.96*se
cat("lower = ",round(lower,4),"  upper = ",round(upper,4),"\n")

###
### not in the book
detach(dframe)
###

# input data contained in x0
jack <- function(x0,n) {
  x <- matrix(rep(x0,length(x0)),n,n)
  matrix(x[row(x)!=col(x)],n-1,n)
}

# test array = a
a <- matrix(16:1,4,4)
a
row(a)==col(a) # diagonal elements
a[row(a)==col(a)]
row(a)!=col(a) # off-diagonal elements
a[row(a)!=col(a)]
row(a)>col(a) # lower triangle elements
a[row(a)>col(a)]

n <- 10
x0 <- rnorm(n,4,2)
jdata <- jack(x0,n)
round(x0,2)
round(jdata,2)
jmean <- apply(jdata,2,mean)
cat("mean(jack)=",round(mean(jmean),3),"mean=",round(mean(x0),3),"\n")
cat("var(jack)=",round(sqrt(sum((n-1)*(jmean-mean(jmean))^2)/n),3),"var=",round(sqrt(var(x0)/n),3),"\n")

###
### - R -- not run
sd <- function(x) { sqrt(var(x)) }
###
n <- 10
x <- rnorm(n)
iter <- 2000
temp <- matrix(sample(x,n*iter,replace=T),n,iter)
sx <- apply(temp,2,sd)
# bias estimate
sd(x)-mean(sx)

# atype contains n1=20 observations for type-A individuals
# btype contains n2=20 observations for type-B individuals
iter <- 2000
d <- NULL
for (i in 1:iter) {
  xtemp <- sample(atype,n1,replace=T)
  ytemp <- sample(btype,n2,replace=T)
  d[i] <- mean(xtemp)-mean(ytemp)
}
dbar <- mean(d)
se <- sqrt(var(d))
# distance from 0 in standard deviations
z <- dbar/se
pvalue <- 2*(1-pnorm(abs(z)))
cbind(dbar,se,z,pvalue)
# 95% limits of mean cholesterol levels
quantile(d,c(0.025,0.975))

p <- sum(ifelse(d<0.01,1,0))/iter
# previous parametric t-value = 2.562
pvalue <- 1-pt(2.562,38)
cbind(p,pvalue)

                                        # Fig. 5.8
###
### not in the book

###

# atype contains n1=20 observations for type-A individuals
# btype contains n2=20 observations for type-B individuals
d <- c(atype,btype)
# randomization -- null distribution
iter <- 5000
difbar <- NULL
for (i in 1:iter) {
  v <- sample(1:length(d),length(atype),replace=F)
  difbar[i] <- mean(d[v])-mean(d[-v])
}
quantile(difbar,seq(0,1,0.1))
dmean <- mean(atype)-mean(btype)
p <- sum(ifelse(difbar>dmean,1,0))/iter
cat("observed difference =",round(dmean,3),"\n","p-value (randomization test) =",round(p,4),"\n")

indata <- read.table("bwt.data",header=T)
# changes data frame into a conventional n by k+1 matrix
xtemp <- data.matrix(indata)
n <- nrow(xtemp)
k <- ncol(xtemp)-1
x <- cbind(rep(1,n),xtemp[,-1])
y <- xtemp[,1]
# estimates the regression coefficients
b <- solve(t(x)%*%x)%*%t(x)%*%y
round(b,4)

# calculation of the estimated values
yhat <- x%*%b
# calculation of the residual values
res <- y-yhat
quantile(res,c(0,0.25,0.5,0.75,1))
ssr <- sum(res^2)
s2 <- ssr/(n-k-1)
cat("\n","estimated variance of y =",round(s2,4),"\n")

# variance/covariance array associated with the estimates b
v <- solve(t(x)%*%x)*s2
round(v,8)

ssr <- sum(res^2)
cat("residual sum of squares =",round(ssr,4),"degrees of freedom",n-k-1,"\n")
ss0 <- sum((y-mean(y))^2)
cat("total sum of squares =",round(ss0,4),"degrees of freedom",n-1,"\n")

df1 <- n-k-1
f <- ((ss0-ssr)/k)(ssr/df1)
pvalue <- 1-pf(f,k,df1)
round(cbind(f,pvalue),k)

# squared multiple correlation coefficient
r2 <- cor(yhat,y)^2
cat("\n","squared multiple correlation coefficient =",round(r2,4),"\n")

sb <- sqrt(diag(v))
tvalues <- b/sb
pvalues <- 2*(1-pt(abs(tvalues),n-k-1))
round(cbind(b,sb,tvalues,pvalues),4)

###
### not in the book
gdata <- read.table("gdata.txt",header=T)
names(gdata) <- c("x","y")
###

# gdata = data frame with 28 pairs (x=weeks, y=gain)
otemp <- nls(~y-a*b^x,gdata,start=list(a=3,b=1))
summary(otemp)

                                        # Fig. 5.9
###
### not in the book
yfit <- coef(otemp)[1]*coef(otemp)[2]^gdata$x
with(gdata, plot(y~x,type="l",xlab="weeks of gestation",ylab="weight gain"))
lines(gdata$x,yfit,lty=3)
title(main="Maternal weight gain by weeks of gestation -- n = 13,202")
legend(15,15,c("observed gain","estimated gain"),lty=c(1,3))
###

b <- lsfit(x,log(y))$coef
b
exp(b)

# age, msv, deaths and pyears defined in the data file
xdata <- read.table("ridge.data",header=T)
rate <- deaths*100000/pyears

otemp <- nls(rate~(1+b*msv)*exp(c*age^2),xdata,start=list(b=0.01,c=0.005))
summary(otemp)

rvalues <- function(b,c,msv,age) { rate-(1+b*msv)*exp(c*age^2) }
otemp <- nls(~rvalues(b,c,msv,age),xdata,start=list(b=0.01,c=0.005))

f <- coef(otemp)
rate0 <- (1+f[1]*msv)*exp(f[2]*age^2)
round(t(matrix(rate0,7,8)),2)
round(cor(rate0,rate),3)

ex <- pyears*rate0/100000
x2 <- sum((deaths-ex)^2/ex)
pvalue <- 1-pchisq(x2,54)
cbind(x2,pvalue)

                                        # Fig. 5.10
# a0 and m0 = xy-coordinates
a0 <- seq(min(age),max(age),1)
m0 <- seq(min(msv),max(msv),1)
# surface height = z
z <- outer(a0,m0,function(a0,m0){(1+f[1]*m0)*exp(f[2]*a0^2)})
persp(a0,m0,z,xlab="age",ylab="mSv",zlab="rate",zlim=c(0,1500))
title("Nonlinear fit -- Oak Ridge data")

otemp0 <- ms(~(rate-(1+b*msv)*exp(c*age^2))^2,xdata,start=list(b=0.03,c=0.001))
coef(otemp0)

# exponential distribution data
n <- 50
x <- rexp(n,2)
# maximum likelihood estimate (closed-form)
1/mean(x)
# ms-estimation
edata <- data.frame(x=x)
temp <- ms(~(-n*log(u)+u*sum(x)),edata,start=list(u=2))
coef(temp)
# equivalently, the sum is implicit
temp <- ms((-log(u)+u*x),edata,start=list(u=2))
coef(temp)

# standard normal distribution "data"
n <- 50
x <- rnorm(n)
# maximum likelihood estimates closed-form
mean(x)
((n-1)/n)*var(x)
# ms-estimation
ndata <- data.frame(x=x)
temp <- ms(~(n*log(v)/2+sum(0.5*(x-m)^2/v)),ndata,start=list(m=0,v=1))
coef(temp)
# equivalently, the sum is implicit
temp <- ms(log(v)/2+0.5*(x-m)^2/v,ndata,start=list(m=0,v=1))
coef(temp)

clike <- function(x,n,k,c0,m,s) {
  z <- (c0-m)/s
  -(n-k)*log(s)-0.5*sum(((x-m)/s)^2)+k*log(pnorm(z))
}

# data
x <- c(0.02,0.18,0.06,0.09,0.03,0.04,0.6,0.04,0.02,0.04,0.09,0.10)
# number of censored values
k <- 5
n <- k+length(x)
# limit of detection
c0 <- 0.015
cat("xbar =",round(mean(x),4),"estimated variance =",round(var(x),4),"\n")
# maximum likelihood estimation
xin <- data.frame(x=x)
est <- ms(-clike(x,n,k,c0,m,s),list(m=mean(x),s=sqrt(var(x))),data=xin)
cat("estimated mean =",round(est$parameters[1],4),"\n","estimated standard deviation =",round(est$parameters[2],4),"\n")

# coffee consumption data
n <- c(41,213,127,142,67,211,133,76)
r <- c(9,94,53,60,11,59,53,28)
# S-likelihood function
like <- function(n,r,a,b1,b2) {
  x1 <- c(0,2,4,5,0,2,4,5)
  x2 <- c(1,1,1,1,0,0,0,0)
  q <- a+b1*x1+b2*x2
  -(sum((n-r)*q)+sum(n*log(1+exp(-q))))
}
datain <- data.frame(r=r,n=n)
ms(~-like(n,r,a,b1,b2),list(a=-2.0,b1=0.5,b2=0.5),data=datain)
# repeat of the glm/logistic analysis
p <- r/n
x1 <- c(0,2,4,5,0,2,4,5)
x2 <- c(1,1,1,1,0,0,0,0)
otemp <- glm(p~x1+x2,family=binomial,weights=n)
otemp$coefficients
