# Modern Applied Biostatistical Methods Using S-PLUS
# S. Selvin, Oxford, 1998
#
# Chapter 7: Analysis of Variance and Some Other S-Functions
#
# Time-stamp: <2008-03-23 22:36:54 chl>

dframe <- read.table("bwt.data",header=T)
attach(dframe)
summary(dframe)

gcuts <- cut(gain,c(-99,11,16,22,99),
             labels=c("small","normal","moderate","extreme"))
cbind(tapply(bwt,gcuts,length),tapply(bwt,gcuts,mean),tapply(bwt,gcuts,var))

gain0 <- factor(gcuts)
otemp <- aov(bwt~gain0,dframe)
otemp

summary(otemp)

                                        # Fig. 7.1
###
### not in the book
par(mfrow=c(1,2))
###
plot.design(otemp,main="Birth weight data")
plot.design(otemp,fun=median,main="Birth weight data")

                                        # Fig. 7.2
par(mfrow=c(1,2))
plot(otemp,main="Birth weight data")

                                        # Fig. 7.3
qqnorm(resid(otemp),main="Birth weight data -- qnorm")
qqline(resid(otemp))

kruskal.test(bwt,gain0)

###
### not in the book
detach(dframe)
###

y <- c(0.12,0.15,0.12,0.13,0.12,0.11,0.12,0.15,0.11,0.12,0.15,0.16)*100
x1 <- c(1,1,1,1,2,2,2,2,3,3,3,3)
x2 <- c(1,1,2,2,3,3,4,4,5,5,6,6)
ydata <- data.frame(y=y,refineries=factor(x1),days=factor(x2))
summary(ydata)

otemp <- aov(y~refineries+refineries/days,ydata)
summary(otemp)

aov(y~refineries+days%in%refineries,ydata)
aov(y~refineries/days,ydata)

y <- c(20,26,28,34,32,38,54,61,68,38,40,46,19,25,31,40,38,54)
x1 <- rep(c(1,2,3),6)
x2 <- rep(1:6,rep(3,6))
methods <- factor(x1)
methods
persons <- factor(x2)
persons
ldata <- data.frame(y=y,methods=methods,persons=persons)
summary(ldata)

otemp1 <- aov(y~methods+persons,ldata)
summary(otemp1)

otemp0 <- aov(y~persons,ldata)
summary(otemp0)

anova(otemp0,otemp1)

                                        # Fig. 7.4
par(mfrow=c(2,1))
interaction.plot(persons,methods,y,main="Lead determination data")
interaction.plot(methods,persons,y)

                                        # Fig. 7.5
###
### not in the book
par(mfrow=c(2,1))
###
plot.factor(ldata,main="Lead determination data")

                                        # Fig. 7.6
qqnorm(resid(otemp1),main="Lead determination data -- qqnorm plot")
qqline(resid(otemp1))

                                        # Fig. 7.7
plot(resid(otemp1),fitted(otemp1),
     main="Lead determination data -- residual plot")

ymean <- tapply(y,methods,mean)
cat("column means",round(ymean,3),"\n")

df <- otemp1$df.residual
mse <- sum(otemp1$residuals^2)/df

T.stat <- function(xbar,ybar,mse,df) {
  tstat <- (xbar-ybar)/sqrt(mse)
  pvalue <- 2*(1-pt(abs(tstat),df))
  cat("tstat = ",round(tstat,3),"   pvalue = ",round(pvalue,3),"\n")
}
T.stat(ymean[1],ymean[2],mse,df)
T.stat(ymean[1],ymean[3],mse,df)
T.stat(ymean[2],ymean[3],mse,df)

friedman.test(y,methods,persons)

otemp <- aov(y~methods,ldata)
summary(otemp)

###
### not in the book
tmp <- read.table("bwt3.txt",header=T)
low <- tmp$ppwt[tmp$bwt==1]
normal <- tmp$ppwt[tmp$bwt==0]
id <- rep(1:24,each=2)
attach(tmp)
###
t.test(low,normal,paired=T)

t.test(low-normal,mu=0)

otemp <- aov(gain~factor(id)+bwt)
summary(otemp)

wilcox.text(low,normal,exact=F,paired=T)

###
### not in the book
detach(tmp)
###

###
### not in the book
attach(dframe)
###
wcut <- c(-99,50,55,72,99)
gcut <- c(-99,11,16,22,99)
gain0 <- cut(gain,gcut,label=c("small","normal","moderate","extreme"))
wt0 <- cut(wt,wcut,label=c("light","normal","heavy","obese"))
tapply(bwt,list(wt0,gain0),length)
round(tapply(bwt,list(wt0,gain0),mean),3)
se <- function(x) {sqrt(var(x)/length(x))}
round(tapply(bwt,list(wt0,gain0),se),3)

otemp0 <- aov(bwt~wt0+gain0+wt0*gain0)
sumamry(otemp0)
otemp1 <- aov(bwt~wt0+gain0)
summary(otemp1)
otemp2 <- aov(bwt~gain0)
summary(otemp2)
otemp3 <- aov(bwt~wt0)
summary(otemp3)

anova(otemp3,otemp1,otemp0)

anova(otemp2,otemp1,otemp0)

otemp <- glm(bwt~ht+wt+ht^2+wt^2+wt*ht)
step(otemp,trace=F)

###
### not in the book
detach(dframe)
rm(list=ls())
###

vnames <- c("ht","wt","ht^2","wt^2","ht*wt")
d <- matrix(scan("bwt.dat"),4)
y <- d[1,]
x1 <- d[2,]
x2 <- d[3,]
x <- cbind(x1,x2,x1^2,x2^2,x1*x2)
otemp <- leaps(x,y,method="Cp",nbest=31,names=vnames)
summary(otemp)
cbind(otemp$label,otemp$size,round(otemp$Cp,3))

                                        # Fig. 7.8
plot(otemp$size,otemp$Cp,pch=".",type="b",
     main="Birth weight data -- model selection")

y <- d[1,]
x1 <- d[2,]
x2 <- d[3,]
x3 <- d[4,]
x <- cbind(x1,x2,x3,x1^2,x2^2,x3^2,x1*x2,x1*x3,x2*x3,x1*x2*x3)
vnames <- c("ht","wt","gain","ht^2","wt^2","gain^2",
            "ht*wt","ht*gain","wt*gain","wt*ht*gain")
otemp <- leaps(x,y,method="Cp",nbest=31,names=vnames)
cp <- otemp$Cp
ordcp <- order(cp)
lmin <- otemp$label[ordcp]
smin <- otemp$size[ordcp]
cp <- sort(cp)
cbind(lmin[1:5],smin[1:5],round(cp[1:5],3))


                                        # Fig. 7.9
###
### not in the book
par(mfrow=c(1,2))
plot(otemp$size,otemp$Cp,pch=".",type="b",ylim=c(0,80),xlim=c(2,11),
     main="Birth weight data -- model selection")
plot(otemp$size,otemp$Cp,pch=".",type="b",ylim=c(0,25),xlim=c(5,8),
     main="Birth weight data -- model selection")
###

new.x <- scale(x,center=T)

###
### not in the book
lbwt <- read.csv("lbwt.csv",header=T,sep=",")
lbwt.data <- lbwt[,2:9]
### Note:
### An error probably occured when writing lbwt.txt as there is some
### variation between Selvin's PCA results and mine.
### Sign reversion also indicates that S-Plus and R implementation of
### prcomp() differ with respect to the (arbitrary) determination of
### principal axes orientation.
###
bdata <- scale(lbwt.data,center=T)
p.object <- prcomp(bdata)
summary(p.object)
v <- p.object$sdev^2
round(v,2)
round(100*v/sum(v),1)
round(100*cumsum(v)/sum(v),1)

round(p.object$rotation,2)

p <- sweep(p.object$x,2,apply(p.object$x,2,mean))
round(p[,1],3)
round(p[,2],3)
ptemp <- cbind(p[,1],p[,2])
round(apply(ptemp,2,mean),3)
round(apply(ptemp,2,var),3)
round(cor(ptemp),3)

round(cor(x,p),3)

                                        # Fig. 7.10
plot(p[,1],p[,2],pch="+",main="First and second principal components")

otemp <- lm(bwt~p[,1]+p[,2])
summary(otemp)

###
### not in the book
cereal <- read.csv("cereal.csv",header=TRUE,sep=",")
cereal <- cereal[,2:13]
###
v <- prcomp(cereal)$sd^2
round(v,2)
round(100*v/sum(v),1)
round(cumsum(100*v/sum(v)),1)

round(prcomp(cereal)$rotation,2)

                                        # Fig. 7.11
p <- prcomp(cereal)$x
plot(p[,1],p[,2],pch="*",main="Cereal classification by principal components")
text(p[,1],p[,2],1:nrow(dtemp),adj=2,cex=0.5)

###
### not in the book
a <- read.table("exlife-urban2.txt",header=T)
attach(a)
### Note:
### An error probably occured when writing lbwt.txt as there is some
### variation between Selvin's PCA results and mine.
###

y <- cbind(exm,exf,births,deaths)
y <- scale(y,center=T)
x <- cbind(urban,income,taxes,density)
x <- scale(x,center=T)
data <- cbind(y,x)
round(cor(data),3)

c.object <- cancor(x,y)
summary(c.object)

c.object$cor

ya <- c.object$ycoef
round(ya,3)
xa <- c.object$xcoef
round(xa,3)

cy <- y%*%ya
round(cor(cbind(cy,y)),3)

p <- ncol(x)
q <- ncol(y)
n <- nrow(data)
mm <- cor(data)
am <- mm[(1:p),(1:p)]
bm <- mm[(p+1):(p+q),(p+1):(p+q)]
cm <- mm[(1:p),(p+1):(p+q)]
m <- solve(bm)%*%t(cm)%*%solve(am)%*%cm
e <- eigen(m)$values
x2 <- -(n-(p+q+1)/2)*sum(log(1-e))
pvalue <- 1-pchisq(x2,p*q)
round(cbind(x2,pvalue),3)
  
###
### not in the book
detach(a)
###
